package org.ak47.android.graphics.ibnrushd;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import org.ak47.android.graphics.ibnrushd.object.IbnRushdObject;
import android.content.Context;
import android.opengl.GLSurfaceView;
import android.opengl.GLSurfaceView.Renderer;
import android.view.View;

public class IbnRushdRenderer implements Renderer
{
	/** Singleton */
	private static IbnRushdRenderer instance = null;
	
	/** OpenGL view */
	private static GLSurfaceView glView = null;
	
	/** The objects to render and how many */
	private static List<IbnRushdObject> objectQueue = null;
	private static int lastObject = -1;
	
	/** 
	 * Lock to prevent modification of the list 
	 * when queueing new objects 
	 */
	private final ReentrantLock lock = new ReentrantLock();
	
	/** View updater */
	private static IbnRushdUpdater updater = null;
	
	private IbnRushdRenderer() {}
	
	protected static void initialize(Context context, IbnRushdUpdater updater)
	{		
		glView = new GLSurfaceView(context);
		instance = new IbnRushdRenderer();
		glView.setRenderer(instance);
		IbnRushdRenderer.updater = updater;
	}
	
	protected static View getView()
	{
		return glView;
	}
	
	protected static int add(IbnRushdObject object)
	{
		if (lastObject == -1)
		{
			objectQueue = new ArrayList<IbnRushdObject>();
		}
		
		objectQueue.add(object);
		lastObject++;
		
		return lastObject;
	}

	@Override
	public void onDrawFrame(GL10 gl)
	{
		if (updater != null)
		{
			updater.onUpdate();
		}
		
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
		try
		{
			lock.lock();
			for(IbnRushdObject object : objectQueue)
			{
				object.executeTransformations(gl);
				object.draw(gl);
			}
		}
		finally
		{
			lock.unlock();
		}
	}

	@Override
	public void onSurfaceChanged(GL10 gl, int width, int height)
	{
		gl.glViewport(0, 0, width, height);
		float aspectRatio;
		float zNear =.1f;
		float zFar =1000;
		float fieldOfView = 30.0f/57.3f;
		float size;
		
		gl.glEnable(GL10.GL_NORMALIZE);
		aspectRatio=(float)width/(float)height;
		gl.glMatrixMode(GL10.GL_PROJECTION);
		size = zNear * (float)(Math.tan((double)(fieldOfView/2.0f)));
		
		gl.glFrustumf(-size, size, -size /aspectRatio, size /aspectRatio, zNear, zFar);
	}

	@Override
	public void onSurfaceCreated(GL10 gl, EGLConfig config)
	{
		gl.glDisable(GL10.GL_DITHER);
		gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT, GL10.GL_NICEST);
		gl.glClearColor(1,1,1,1);
		gl.glEnable(GL10.GL_CULL_FACE);
		gl.glShadeModel(GL10.GL_SMOOTH);
		gl.glEnable(GL10.GL_DEPTH_TEST);
	}
	
	public static void onResume()
	{
		if (glView != null)
		{
			glView.onResume();
		}
	}
	
	public static void onPause()
	{
		if (glView != null)
		{
			glView.onPause();
		}
	}
}
