package org.ak47.android.graphics.ibnrushd.object;

import java.util.Iterator;
import javax.microedition.khronos.opengles.GL10;

import org.ak47.android.graphics.ibnrushd.mesh.IbnRushdMesh;
import org.ak47.android.graphics.ibnrushd.transformation.IbnRushdTransformation;
import org.ak47.android.graphics.ibnrushd.transformation.IbnRushdTransformationList;

/**
 * Modelizes a visual object, which can be acted upon with transformations (IbnRushdTransformation)
 * @author m0skit0
 */
public class IbnRushdObject
{
	/** The graphical object to show */
	private IbnRushdMesh mesh = null;
	
	/** Sum of total transformations */
	private IbnRushdObjectTransformation totalTransformation = new IbnRushdObjectTransformation();
	
	/** List of transformations to be executed upon this object */
	private IbnRushdTransformationList transformations;
	
	/**
	 * Initializes this IbnRushd object with a mesh
	 * @param mesh
	 */
	public IbnRushdObject(IbnRushdMesh mesh)
	{
		this.mesh = mesh;
	}
	
	/**
	 * Adds a transformation to be performed on this object<br>
	 * Any previous transformation of this type is overwritten<br>
	 * The transformation does not take place until {@link #executeTransformations(GL10)} is called
	 * @param trans
	 */
	public synchronized void addTransformation(IbnRushdTransformation trans)
	{
		if (transformations == null)
		{
			transformations = new IbnRushdTransformationList();
		}
		transformations.add(trans);
	}
	
	/**
	 * Returns current transformation of this type for this object; <i>null</i> if no transformation of this type are defined on this object  
	 * @param type
	 * @return
	 */
	public synchronized IbnRushdTransformation getTransformation(int type)
	{
		IbnRushdTransformation ret = null;
		if (transformations != null)
		{
			ret = transformations.getTransformation(type);
		}
		return ret;
	}
	
	/**
	 * Calculates total transformation on this object
	 * @param gl
	 */
	public synchronized void executeTransformations(GL10 gl)
	{
		if (transformations != null)
		{
			if (transformations.size() > 0)
			{
				// For all transformations defined for this object
				Iterator<IbnRushdTransformation> iterator = transformations.getIterator();
				while(iterator.hasNext())
				{
					IbnRushdTransformation trans = iterator.next();
					// Apply this transformation to the global transformation
					trans.applyTo(totalTransformation);
				}
			}
		}
	}
	
	/**
	 * Executes total transformation and draws the object
	 * @param gl
	 */
	public void draw(GL10 gl)
	{
		// Execute all transformations on object
		totalTransformation.execute(gl);
		
		// Draw mesh
		mesh.draw(gl);
	}
}