package org.ak47.android.graphics.ibnrushd.transformation;

import org.ak47.android.graphics.ibnrushd.object.IbnRushdObjectTransformation;

import android.opengl.Matrix;

/**
 * Defines a virtual transformation<br>
 * All transformations must inherit from this class
 * @author m0skit0
 */
public abstract class IbnRushdTransformation
{
	/** How many transformations are available */
	public final static int NUM_TRANSFORMATIONS = 3;
	
	/** Types of transformation */
	public final static int NO_TYPE = -1;
	public final static int ROTATION = 0;
	public final static int TRANSLATION = 1;
	public final static int SCALING = 2;
		
	/** This subclass transformation type */
	protected static int TYPE = NO_TYPE;
	
	/** This transformation matrix */
	protected float[] mTransformation = new float[16];
	
	public IbnRushdTransformation()
	{
		Matrix.setIdentityM(mTransformation, 0);
	}
	
	public int getType()
	{
		return TYPE;
	}
	
	public float[] getTransformationMatrix()
	{
		return mTransformation;
	}
	
	/** Applies this tranformation to a final transformation */
	public void applyTo(IbnRushdObjectTransformation t)
	{
		float[] mT = t.getMatrix();
		float[] mCompos = new float[16];
		
		// mCompos = mT x mTransformation
		Matrix.multiplyMM(mCompos, 0, mT, 0, mTransformation, 0);
		
		t.setMatrix(mCompos);
	}
}
