package org.ak47.android.graphics.ibnrushd.transformation;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Holds a list of transformations<br>
 * There cannot be more than one transformation of one type<br>
 * NOT multi-thread safe
 * @author m0skit0
 */
public class IbnRushdTransformationList implements Iterator<IbnRushdTransformation>
{
	/** Lists the transformations by type */
	private IbnRushdTransformation[] transformations = new IbnRushdTransformation[IbnRushdTransformation.NUM_TRANSFORMATIONS];
	
	/** Lists the order of transformations */
	private List<Integer> list = new LinkedList<Integer>();
	
	/** Iterator for list */
	private Iterator<Integer> listIterator;
	
	/** 
	 * Returns the iterator for this list<br>
	 * Equivalent to List.iterator()
	 * @return
	 */
	public synchronized Iterator<IbnRushdTransformation> getIterator()
	{
		listIterator = list.iterator();
		return this;
	}
	
	/**
	 * Adds a transformation to the list<br>
	 * If the transformation type already exists, replace it
	 * @param t
	 */
	public synchronized void add(IbnRushdTransformation t)
	{
		int type = t.getType();
		if (!list.contains(type))
		{
			list.add(type);
		}
		transformations[type] = t;
	}
	
	/**
	 * Returns this list's size
	 * @return
	 */
	public synchronized int size()
	{
		if (list == null)
		{
			return 0;
		}
			
		return list.size();
	}

	@Override
	public synchronized boolean hasNext()
	{
		return listIterator.hasNext();
	}

	@Override
	public synchronized IbnRushdTransformation next()
	{
		return transformations[listIterator.next()];
	}

	@Override
	public synchronized void remove()
	{
		listIterator.remove();		
	}
	
	/**
	 * Returns this transformation type from the list 
	 * @param type
	 * @return
	 */
	public synchronized IbnRushdTransformation getTransformation(int type)
	{
		return transformations[type];
	}
}